﻿class Person {
	public string FirstName { get; set; }
	public string LastName { get; set; }
	public int Age { get; set; }
}

class Program {
	static (string name, int age) GetInfo() {
		var localName = Console.ReadLine();
		if (!int.TryParse(Console.ReadLine(), out var localAge)) {
			localAge = -1;
		}

		return (localName, localAge);
	}

	static IEnumerable<(int id, string name)> GetList() {
		yield return (id: 1, name: "one");
		yield return (2, "two");
	}

	static void Main(string[] args) {
		var info = GetInfo();
		Console.WriteLine(info.name);
		Console.WriteLine(info.age);

		foreach (var x in GetList()) {
			Console.WriteLine(x.id);
			Console.WriteLine(x.name);
		}

		var (personName, personAge) = GetInfo();
		Console.WriteLine(personName);
		Console.WriteLine(personAge);

		(personName, personAge) = GetInfo();
		Console.WriteLine(personName);
		Console.WriteLine(personAge);

		string someString = "Donald";
		int someInt = 42;
		Console.WriteLine(someString);
		Console.WriteLine(someInt);

		(someString, someInt) = GetInfo();
		Console.WriteLine(someString);
		Console.WriteLine(someInt);

		// (string personName2, string personAge2) = GetInfo();
		// Error: Cannot implicitly convert type 'int' to 'string'

		(string personName2, long personAge2) = GetInfo();
		Console.WriteLine(personName2);
		Console.WriteLine(personAge2);

		(var theName, _) = GetInfo();
		Console.WriteLine(theName);

		// You cannot have the following variable declared in a same scope, if using the _ symbol as wildcard!
		// var _ = "Hello";

		(_, var theAge) = GetInfo();
		Console.WriteLine(theAge);

		var person = new Person { FirstName = "John", LastName = "Doe", Age = 42 };

		var tuple = (person.FirstName, person.Age);
		Console.WriteLine(tuple.Item1);
		Console.WriteLine(tuple.Item2);

		var person2 = new DecomposablePerson { FirstName = "John", LastName = "Doe", Age = 42 };

		(var name, _, var age) = person2;
		Console.WriteLine(name);
		Console.WriteLine(age);
	}
}

class DecomposablePerson {
	public string FirstName { get; set; }
	public string LastName { get; set; }
	public int Age { get; set; }

	public void Deconstruct(out string firstName, out string lastName, out int age) {
		firstName = FirstName;
		lastName = LastName;
		age = Age;
	}
}
